#========================================================================================
# package      DE_EPAGES::Moneybookers::t::unit::UI_BasketOrder
# base         DE_EPAGES::Test::API::UnitTest::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::BasketOrder
#========================================================================================
package DE_EPAGES::Moneybookers::t::unit::UI_BasketOrder;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Moneybookers::UI::BasketOrder;
use Test::MockObject::Extends;
use Test::Exception;
use Test::More;

use strict;

use constant SHORT_FORMATTED_CURRENT_DATE_TIME => '2017-11-07T08:54:21+06:00';
use constant STATUS_RETURN_BUILT_URL => 'https://fakebuilturl.epages.com';
use constant MONEYBOOKERS_PROVIDER_ID => '1435672';
use constant MONEYBOOKERS_ACCOUNT_EMAIL => 'test@epages.com';
use constant MONEYBOOKERS_PAYMENT_METHOD_CODE => 'ACI';
use constant LINE_ITEM_CONTAINER_ID => '34521';
use constant LINE_ITEM_CONTAINER_GRAND_TOTAL => 32.56;
use constant LINE_ITEM_CONTAINER_CURRENCY_ID => 'EUR';
use constant HIRE_PURCHASE_INSTALLMENT_AMOUNT => 12.33;
use constant HIRE_PURCHASE_PERIOD_UNIT => 'month';
use constant HIRE_PURCHASE_NUMBER_OF_UNITS => 5;

sub test {
    my $self = shift;

    $self->prepareCommonMocks();

    $self->test_ViewPaymentMoneybookers_servlet_vars();
    $self->test_ViewPaymentMoneybookers_servlet_vars_for_hire_purchase();

    done_testing();
}

sub test_ViewPaymentMoneybookers_servlet_vars {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $Shop = $self->memory->get('UnityShop');
    my $Order = $self->_buildCustomerOrder({
        'IsHirePurchase' => 0,
        'Shop' => $Shop,
    });
    my $Servlet = $self->_buildServlet({
        'Object' => $Order
    });
    my $SUT = $self->_buildBasketOrderUI();

    ## WHEN
    $SUT->ViewPaymentMoneybookers($Servlet);

    ## THEN
    is($Servlet->vars('IsHirePurchase'), 0, "$SubTestName - verify Servlet holds 'IsHirePurchase' parameter as zero.");
    is($Servlet->vars('MoneybookersProviderID'), MONEYBOOKERS_PROVIDER_ID, "$SubTestName - verify Servlet 'MoneybookersProviderID' parameter.");
    is($Servlet->vars('PaymentMethodEmail'), MONEYBOOKERS_ACCOUNT_EMAIL, "$SubTestName - verify Servlet 'PaymentMethodEmail' parameter.");
    is($Servlet->vars('PaymentMethodCode'), MONEYBOOKERS_PAYMENT_METHOD_CODE, "$SubTestName - verify Servlet 'PaymentMethodCode' parameter.");
    like($Servlet->vars('TransactionId'), qr/^unityTestShop\.*/, "$SubTestName - verify Servlet 'TransactionId' parameter.");
    is($Servlet->vars('LineItemContainerID'), LINE_ITEM_CONTAINER_ID, "$SubTestName - verify Servlet 'LineItemContainerID' parameter.");
    is($Servlet->vars('GrandTotal'), LINE_ITEM_CONTAINER_GRAND_TOTAL, "$SubTestName - verify Servlet 'GrandTotal' parameter.");
    is($Servlet->vars('CurrencyID'), LINE_ITEM_CONTAINER_CURRENCY_ID, "$SubTestName - verify Servlet 'CurrencyID' parameter.");
    is($Servlet->vars('ReturnURL'), STATUS_RETURN_BUILT_URL, "$SubTestName - verify Servlet 'ReturnURL' parameter.");
}

sub test_ViewPaymentMoneybookers_servlet_vars_for_hire_purchase {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    ## GIVEN
    my $Shop = $self->memory->get('UnityShop');
    my $Order = $self->_buildCustomerOrder({
        'IsHirePurchase' => 1,
        'Shop' => $Shop,
    });
    my $Servlet = $self->_buildServlet({
        'Object' => $Order
    });
    my $SUT = $self->_buildBasketOrderUI();

    ## WHEN
    $SUT->ViewPaymentMoneybookers($Servlet);

    ## THEN
    is($Servlet->vars('IsHirePurchase'), 1, "$SubTestName - verify Servlet holds 'IsHirePurchase' parameter as one.");
    is($Servlet->vars('HirePurchaseInstallmentAmount'), HIRE_PURCHASE_INSTALLMENT_AMOUNT, "$SubTestName - verify Servlet 'HirePurchaseInstallmentAmount' parameter.");
    is($Servlet->vars('HirePurchaseInstallmentPeriodUnit'), HIRE_PURCHASE_PERIOD_UNIT, "$SubTestName - verify Servlet 'HirePurchaseInstallmentPeriodUnit' parameter.");
    is($Servlet->vars('HirePurchaseInstallmentPeriodAmount'), 1, "$SubTestName - verify Servlet 'HirePurchaseInstallmentPeriodAmount' parameter.");

}

#===============================================================
# Mocks
#===============================================================
sub prepareCommonMocks {
    my $self = shift;

    my $UnityShop = $self->mockFactory->buildShop({
        'IsUnityRegistered' => 1,
        'NameOrAlias' => 'unityTestShop'
    });

    my $Epages6Shop = $self->mockFactory->buildShop({
        'IsUnityRegistered' => 0
    });

    $self->memory->add('UnityShop', $UnityShop->getMockObject());
    $self->memory->add('Ep6Shop', $Epages6Shop->getMockObject());

    return;
}


sub _buildBasketOrderUI {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Moneybookers::UI::BasketOrder', []);
    $Mockify->addMockWithReturnValue('_getFormattedCurrentDateTime', SHORT_FORMATTED_CURRENT_DATE_TIME);
    $Mockify->addMockWithReturnValue('_loadRootObject', $self->_buildSystem());

    my $aBuildShopUrlParameterTypes = ['object', 'hashref', 'hashref'];
    $Mockify->addMockWithReturnValueAndParameterCheck('_buildShopUrl', STATUS_RETURN_BUILT_URL, $aBuildShopUrlParameterTypes);

    my $aLogRequestParameterTypes = [
            {'object' => 'DE_EPAGES::Presentation::Servlet::Permission'},
            {'object' => 'DE_EPAGES::Shop::API::Object::Shop'},
            'string',
            'hashref'
    ];
    $Mockify->addMockWithReturnValueAndParameterCheck('_logRequest', undef, $aLogRequestParameterTypes);

    return $Mockify->getMockObject();
}

sub _buildPaymentMethod {
    my $self = shift;

    my $hParameters = {
        'Email' => MONEYBOOKERS_ACCOUNT_EMAIL,
        'SendStatusEmail' => 0,
        'SplitGatewayMethod' => MONEYBOOKERS_PAYMENT_METHOD_CODE
    };
    my $EpagesObjectMock = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Moneybookers::API::Object::PaymentMethodMoneybookers', $hParameters
    );

    return $EpagesObjectMock->getMockObject();
}

sub _buildLineItemPayment {
    my $self = shift;
    my ($PaymentErrorCode, $IsHirePurchase) = @_;

    my $hParameters = {
        'HirePurchaseInstallmentAmount' => ($IsHirePurchase) ? HIRE_PURCHASE_INSTALLMENT_AMOUNT : undef,
        'HirePurchaseNoOfMonths' => ($IsHirePurchase) ? HIRE_PURCHASE_NUMBER_OF_UNITS : undef,
        'IsHirePurchaseAllowed' => ($IsHirePurchase) ? 1 : 0,
        'PaymentMethod' => $self->_buildPaymentMethod(),
        'PaymentErrorCode' => $PaymentErrorCode,
    };
    my $EpagesObjectMock = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Moneybookers::API::Object::LineItemPaymentMoneybookers', $hParameters
    );

    return $EpagesObjectMock->getMockObject();
}

sub _buildLineItemContainer {
    my $self = shift;
    my ($PaymentErrorCode, $IsHirePurchase) = @_;

    my $hParameters = {
        'CurrencyID' => LINE_ITEM_CONTAINER_CURRENCY_ID,
        'GrandTotal' => LINE_ITEM_CONTAINER_GRAND_TOTAL,
        'ID' => LINE_ITEM_CONTAINER_ID,
        'Payment' => $self->_buildLineItemPayment($PaymentErrorCode, $IsHirePurchase)
    };
    my $EpagesObjectMock = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::LineItemContainer', $hParameters
    );

    return $EpagesObjectMock->getMockObject();
}

sub _buildCustomerOrder {
    my $self = shift;
    my ($hProperties) = @_;

    my $hParameters = {
        'instanceOf' => 'DE_EPAGES::Order::API::Object::CustomerOrder',
        'Shop' => $hProperties->{'Shop'},
        'WebUrlUnitySSL' => $hProperties->{'WebUrlUnitySSL'},
        'LineItemContainer' => $self->_buildLineItemContainer($hProperties->{'PaymentErrorCode'}, $hProperties->{'IsHirePurchase'})
    };
    my $EpagesObjectMock = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::CustomerOrder', $hParameters
    );

    return $EpagesObjectMock->getMockObject();
}

sub _buildServlet {
    my $self = shift;
    my ($hProperties) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($hProperties->{'Object'});
    $MockifyServlet->mock('redirect', undef, ['string']);

    return $MockifyServlet->getMockObject();
}

sub _buildSystem {
    my $self = shift;

    my $hParameters = {
        'Moneybookers_ProviderID' => MONEYBOOKERS_PROVIDER_ID
    };
    my $EpagesObjectMock = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::Site', $hParameters
    );

    return $EpagesObjectMock->getMockObject();
}

__PACKAGE__->runScript();