#========================================================================================
# package      DE_EPAGES::Saferpay2::UI::BasketOrder
# base         DE_EPAGES::Presentation::UI::Object
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for basket or order
#========================================================================================
package DE_EPAGES::Saferpay2::UI::BasketOrder;
use base DE_EPAGES::Presentation::UI::Object;

use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;
use DE_EPAGES::Saferpay2::API::SaferpayService;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Saferpay2::API::Constants qw ( PAYMENT_TYPE_SAFERPAY2 );
use DE_EPAGES::Saferpay2::API::Tools qw ( HandleTransactionResponse );

use strict;



#========================================================================================
# function     ViewPaymentSaferpay2
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewPaymentSaferpay2( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller for classical/mobile checkout
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewPaymentSaferpay2 {
    my $self    = shift;
    my ($Servlet) = @_;

    my $Order  = $Servlet->object();
    my $LineItemPayment = $Order->get('LineItemContainer')->get('Payment');
    my $PaymentMethod   = $LineItemPayment->get('PaymentMethod');

    my $PaymentWebhookUrl = $self->_buildReturnURL(
        $Servlet,
        $Order,
        $LineItemPayment,
        {'ChangeAction' => 'PaymentCallbackSaferpay2'}
    );

    my $ShopperRedirectUrl = $self->_buildReturnURL(
        $Servlet,
        $Order,
        $LineItemPayment,
        {'ChangeAction' => 'PaymentBacklinkSaferpay2'}
    );

    my $SaferpayService = DE_EPAGES::Saferpay2::API::SaferpayService->new($LineItemPayment);
    my $hResponse = $SaferpayService->InitializePaymentPage($Order, $PaymentWebhookUrl, $ShopperRedirectUrl);
    $LineItemPayment->set({'IsTestTransaction' => $PaymentMethod->get('IsInTestMode')});

    $hResponse = HandleTransactionResponse($hResponse);

    if ( $hResponse->{'HasErrorsInResponse'} ) {
        $self->_LogPayment('SHOP ID', {'id' => $Order->getSite()->id()});
        $Servlet->vars('SaferpayPaymentFailed', 1);
        $Servlet->vars('SaferpayPaymentFailedReason', $hResponse->{'ErrorName'});
        $LineItemPayment->updateTransactionStatus(
            $hResponse->{'Transaction'}->{'Id'},
            $hResponse->{'Transaction'}->{'Status'},
            $hResponse->{'Transaction'}->{'ExternalStatus'}
        );

        my $Shop = $Order->getSite();
        if ($Shop->isUnity()) {
            $LineItemPayment->set({'PaymentErrorCode' => 'Failed'});
        }

        $Servlet->redirect($ShopperRedirectUrl);
    } else {
        $LineItemPayment->set( {
            'TransTime'         => GetCurrentDateTime(),
            'TransID2'          => $hResponse->{'Token'}
        });
        $Servlet->vars('SaferpayRedirectUrl', $hResponse->{'RedirectUrl'});
    }
    return;
}


#========================================================================================
# function     ViewMultiPaymentSaferpay2
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewMultiPaymentSaferpay2( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller for multi-step checkout
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewMultiPaymentSaferpay2 {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess', $BasketOrderProcess->tleHash());

    $self->ViewPaymentSaferpay2($Servlet);

    return;
}

sub _buildReturnURL {
    my $self = shift;
    my ( $Servlet, $Order, $LineItemPayment, $hParams ) = @_;
    my $Shop = $Order->get('Shop');

    $hParams->{'OrderId'} = $Order->id();
    $hParams->{'LineItemPaymentGUID'} = $LineItemPayment->guid();
    my $hOptions = {
        'Type' => 'sf/mobile',
        'UseSSL' => 1,
        'AddToken' => 1
    };
    my $URL = BuildShopUrl( $Shop, $hParams, $hOptions, $Servlet );
    return $URL;
}

sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_SAFERPAY2, $Name, $DebugData, $Section, $Filter);
}

1;