#========================================================================================
# package      DE_EPAGES::Saferpay2::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Saferpay2::UI::UI_Shop
#========================================================================================
package DE_EPAGES::Saferpay2::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use Test::Exception;
use Test::More;
use Test::Mockify;
use Test::Mockify::Sut;
use Test::Mockify::Matcher qw ( HashRef String Number );
use Test::Mockify::Verify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled );

use strict;

use constant SAFERPAY_INITIALIZE_TOKEN => 'KJH765765HGFHF';

sub test {
    my $self = shift;

    $self->test_PaymentBacklinkSaferpay2Successful();
    $self->test_PaymentBacklinkSaferpay2Unsuccessful();
    $self->test_PaymentCallbackSaferpay2Successful();
    $self->test_PaymentCallbackSaferpay2Unsuccessful();
    $self->test_CaptureTransactionsSuccessfulAlreadyCaptured();
    $self->test_CaptureTransactionsSuccessful();
    $self->test_CaptureTransactionsUnsuccessfulWrongStatus();
    $self->test_CaptureTransactionsUnsuccessfulAlreadyCaptured();
    $self->test_CaptureTransactionsUnsuccessfulIdNone();
    $self->test_PaymentCallbackSaferpay2SuccessfulWithDirectCapture();
    $self->test_PaymentBacklinkSaferpay2SuccessfulWithDirectCapture();

    done_testing();
}

sub test_PaymentCallbackSaferpay2SuccessfulWithDirectCapture {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedOrder = $self->_createOrder();
    my $PaymentMethod = $MockedOrder->get('LineItemContainer')->get('Payment')->get('PaymentMethod');
    $PaymentMethod->set({'IsDirectCapture' => 1});
    my $MockedShop = $self->_createShop($MockedOrder);
    my $MockedServlet = $self->_createServlet($MockedShop);
    my $MockedSaferpayService = $self->_createSaferpayService();
    my $hHandleTransactionResponse = {
        HasErrorsInResponse => 0,
        Transaction         => {
            Id     => '123',
            Status => 'AUTHORIZED',
            ExternalStatus => 'AUTHORIZED',
        },
        Liability => {
            LiabilityShift => 1
        },
    };
    my $SUT = $self->_createSUT($MockedSaferpayService, $hHandleTransactionResponse);

    $SUT->PaymentBacklinkSaferpay2($MockedServlet);
    my $MockedLineItemPayment = $MockedOrder->get('LineItemContainer')->get('Payment');

    my $aUpdateTransactionStatusParams1stCall = GetParametersFromMockifyCall($MockedLineItemPayment, 'updateTransactionStatus', 0);
    my $aExpectedParameters1stCall = ['123', 'AUTHORIZED', 'AUTHORIZED'];
    is_deeply($aUpdateTransactionStatusParams1stCall, $aExpectedParameters1stCall, "$SubTestName - updateTransactionStatus set authorize parameters ok.");
    my $aUpdateTransactionStatusParams2ndCall = GetParametersFromMockifyCall($MockedLineItemPayment, 'updateTransactionStatus', 1);
    my $aExpectedParameters2ndCall = ['123', 'CAPTURED', 'CAPTURED'];
    is_deeply($aUpdateTransactionStatusParams2ndCall, $aExpectedParameters2ndCall, "$SubTestName - updateTransactionStatus set capture parameters ok.");

    my $aActualUpdateLiabilityParams = GetParametersFromMockifyCall($MockedLineItemPayment, 'updateLiabilityShift');
    my $aExpectedUpdateLiabilityParams = [1];
    is_deeply($aActualUpdateLiabilityParams, $aExpectedUpdateLiabilityParams, "$SubTestName - updateLiabilityShift set parameters ok.");
}


sub test_PaymentBacklinkSaferpay2SuccessfulWithDirectCapture {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedOrder = $self->_createOrder();
    my $PaymentMethod = $MockedOrder->get('LineItemContainer')->get('Payment')->get('PaymentMethod');
    $PaymentMethod->set({'IsDirectCapture' => 1});
    my $MockedShop = $self->_createShop($MockedOrder);
    my $MockedServlet = $self->_createServlet($MockedShop);
    my $MockedSaferpayService = $self->_createSaferpayService();
    my $hHandleTransactionResponse = {
        HasErrorsInResponse => 0,
        Transaction         => {
            Id     => '123',
            Status => 'AUTHORIZED',
            ExternalStatus => 'AUTHORIZED',
        },
        Liability => {
            LiabilityShift => 1
        },
    };
    my $SUT = $self->_createSUT($MockedSaferpayService, $hHandleTransactionResponse);

    $SUT->PaymentCallbackSaferpay2($MockedServlet);
    my $MockedLineItemPayment = $MockedOrder->get('LineItemContainer')->get('Payment');

    my $aUpdateTransactionStatusParams1stCall = GetParametersFromMockifyCall($MockedLineItemPayment, 'updateTransactionStatus', 0);
    my $aExpectedParameters1stCall = ['123', 'AUTHORIZED', 'AUTHORIZED'];
    is_deeply($aUpdateTransactionStatusParams1stCall, $aExpectedParameters1stCall, "$SubTestName - updateTransactionStatus set authorize parameters ok.");
    my $aUpdateTransactionStatusParams2ndCall = GetParametersFromMockifyCall($MockedLineItemPayment, 'updateTransactionStatus', 1);
    my $aExpectedParameters2ndCall = ['123', 'CAPTURED', 'CAPTURED'];
    is_deeply($aUpdateTransactionStatusParams2ndCall, $aExpectedParameters2ndCall, "$SubTestName - updateTransactionStatus set capture parameters ok.");

    my $aActualUpdateLiabilityParams = GetParametersFromMockifyCall($MockedLineItemPayment, 'updateLiabilityShift');
    my $aExpectedUpdateLiabilityParams = [1];
    is_deeply($aActualUpdateLiabilityParams, $aExpectedUpdateLiabilityParams, "$SubTestName - updateLiabilityShift set parameters ok.");
}


sub test_PaymentBacklinkSaferpay2Successful {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedOrder = $self->_createOrder();
    my $MockedShop = $self->_createShop($MockedOrder);
    my $MockedServlet = $self->_createServlet($MockedShop);
    my $MockedSaferpayService = $self->_createSaferpayService();
    my $hHandleTransactionResponse = {
        HasErrorsInResponse => 0,
        Transaction         => {
            Id     => '123',
            Status => 'AUTHORIZED',
            ExternalStatus => 'AUTHORIZED',
        },
        Liability => {
            LiabilityShift => 1
        },
    };
    my $SUT = $self->_createSUT($MockedSaferpayService, $hHandleTransactionResponse);

    $SUT->PaymentBacklinkSaferpay2($MockedServlet);
    my $MockedLineItemPayment = $MockedOrder->get('LineItemContainer')->get('Payment');

    my $aUpdateTransactionStatusParams = GetParametersFromMockifyCall($MockedLineItemPayment, 'updateTransactionStatus');
    my $aExpectedParameters = ['123', 'AUTHORIZED', 'AUTHORIZED'];
    is_deeply($aUpdateTransactionStatusParams, $aExpectedParameters, "$SubTestName - updateTransactionStatus set parameters ok.");

    my $aActualUpdateLiabilityParams = GetParametersFromMockifyCall($MockedLineItemPayment, 'updateLiabilityShift');
    my $aExpectedUpdateLiabilityParams = [1];
    is_deeply($aActualUpdateLiabilityParams, $aExpectedUpdateLiabilityParams, "$SubTestName - updateLiabilityShift set parameters ok.");
}

sub test_PaymentBacklinkSaferpay2Unsuccessful {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedOrder = $self->_createOrder();
    my $MockedShop = $self->_createShop($MockedOrder);
    my $MockedServlet = $self->_createServlet($MockedShop);
    my $MockedSaferpayService = $self->_createSaferpayService();
    my $hHandleTransactionResponse = {
        HasErrorsInResponse => 1,
        Transaction         => {
            Id     => 'NONE',
            Status => 'FAILED',
            ExternalStatus => 'TRANSACTION_DECLINED',
        }
    };
    my $SUT = $self->_createSUT($MockedSaferpayService, $hHandleTransactionResponse);

    $SUT->PaymentBacklinkSaferpay2($MockedServlet);
    my $MockedLineItemPayment = $MockedOrder->get('LineItemContainer')->get('Payment');
    my $aUpdateTransactionStatusParams = GetParametersFromMockifyCall($MockedLineItemPayment, 'updateTransactionStatus');
    my $aExpectedParameters = ['NONE', 'FAILED', 'TRANSACTION_DECLINED'];
    is_deeply($aUpdateTransactionStatusParams, $aExpectedParameters, "$SubTestName - updateTransactionStatus set parameters ok.");

    is(WasCalled($MockedLineItemPayment , 'updateLiabilityShift'), 0, "$SubTestName - tests if updateLiabilityShift was not called.");
}

sub test_PaymentCallbackSaferpay2Successful {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedOrder = $self->_createOrder();
    my $MockedShop = $self->_createShop($MockedOrder);
    my $MockedServlet = $self->_createServlet($MockedShop);
    my $MockedSaferpayService = $self->_createSaferpayService();
    my $hHandleTransactionResponse = {
        HasErrorsInResponse => 0,
        Transaction         => {
            Id     => '123',
            Status => 'AUTHORIZED',
            ExternalStatus => 'AUTHORIZED',
        },
        Liability => {
            LiabilityShift => 1
        },
    };
    my $SUT = $self->_createSUT($MockedSaferpayService, $hHandleTransactionResponse);

    $SUT->PaymentCallbackSaferpay2($MockedServlet);
    my $MockedLineItemPayment = $MockedOrder->get('LineItemContainer')->get('Payment');
    my $aUpdateTransactionStatusParams = GetParametersFromMockifyCall($MockedLineItemPayment, 'updateTransactionStatus');
    my $aExpectedParameters = ['123', 'AUTHORIZED', 'AUTHORIZED'];
    is_deeply($aUpdateTransactionStatusParams, $aExpectedParameters, "$SubTestName - updateTransactionStatus set parameters ok.");

    my $aActualUpdateLiabilityParams = GetParametersFromMockifyCall($MockedLineItemPayment, 'updateLiabilityShift');
    my $aExpectedUpdateLiabilityParams = [1];
    is_deeply($aActualUpdateLiabilityParams, $aExpectedUpdateLiabilityParams, "$SubTestName - updateLiabilityShift set parameters ok.");
}

sub test_PaymentCallbackSaferpay2Unsuccessful {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedOrder = $self->_createOrder();
    my $MockedShop = $self->_createShop($MockedOrder);
    my $MockedServlet = $self->_createServlet($MockedShop);
    my $MockedSaferpayService = $self->_createSaferpayService();
    my $hHandleTransactionResponse = {
        HasErrorsInResponse => 1,
        Transaction         => {
            Id     => 'NONE',
            Status => 'FAILED',
            ExternalStatus => 'TRANSACTION_DECLINED',
        }
    };
    my $SUT = $self->_createSUT($MockedSaferpayService, $hHandleTransactionResponse);

    $SUT->PaymentCallbackSaferpay2($MockedServlet);
    my $MockedLineItemPayment = $MockedOrder->get('LineItemContainer')->get('Payment');
    my $aUpdateTransactionStatusParams = GetParametersFromMockifyCall($MockedLineItemPayment, 'updateTransactionStatus');
    my $aExpectedParameters = ['NONE', 'FAILED', 'TRANSACTION_DECLINED'];
    is_deeply($aUpdateTransactionStatusParams, $aExpectedParameters, "$SubTestName - updateTransactionStatus set parameters ok.");

    is(WasCalled($MockedLineItemPayment , 'updateLiabilityShift'), 0, "$SubTestName - tests if updateLiabilityShift was not called.");
}


sub test_CaptureTransactionsSuccessful {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedTransaction = $self->_createLineItemPaymentSaferpay2('AUTHORIZED', '123');
    my $MockedShop = $self->_createShop($MockedTransaction);
    my $MockedForm = $self->_createForm();
    my $MockedServlet = $self->_createServlet($MockedShop, $MockedForm);
    my $MockedSaferpayService = $self->_createSaferpayService();
    my $SUT = $self->_createSUT($MockedSaferpayService, {});
    $SUT->CaptureTransactions($MockedServlet);
    my $aUpdateTransactionStatusParams = GetParametersFromMockifyCall($MockedTransaction, 'updateTransactionStatus');
    my $aExpectedParameters = [$MockedTransaction->get('TransID'), 'CAPTURED', 'CAPTURED'];
    is_deeply($aUpdateTransactionStatusParams, $aExpectedParameters, "$SubTestName - updateTransactionStatus set parameters ok.");
}


sub test_CaptureTransactionsSuccessfulAlreadyCaptured {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedTransaction = $self->_createLineItemPaymentSaferpay2('AUTHORIZED', '123');
    my $MockedShop = $self->_createShop($MockedTransaction);
    my $MockedForm = $self->_createForm();
    my $MockedServlet = $self->_createServlet($MockedShop, $MockedForm);
    my $hCaptureResponse = {
        ErrorName => 'TRANSACTION_ALREADY_CAPTURED'
    };
    my $MockedSaferpayService = $self->_createSaferpayService($hCaptureResponse);
    my $SUT = $self->_createSUT($MockedSaferpayService, {});
    $SUT->CaptureTransactions($MockedServlet);
    my $aActualFormParams = GetParametersFromMockifyCall($MockedForm, 'addFormError');
    my $hExpectedFormParameters = [{
        Reason => 'ALREADY_CAPTURED',
        TransactionId => $MockedTransaction->get('TransID'),
        LoopName => 'ListObjectIDs',
        Form => 'Transactions',
        Name => 'ListObjectID'
    }];
    is_deeply($aActualFormParams, $hExpectedFormParameters, "$SubTestName - Form error parameters ok.");

    my $aUpdateTransactionStatusParams = GetParametersFromMockifyCall($MockedTransaction, 'updateTransactionStatus');
    my $aExpectedParameters = [$MockedTransaction->get('TransID'), 'CAPTURED', 'CAPTURED'];
    is_deeply($aUpdateTransactionStatusParams, $aExpectedParameters, "$SubTestName - updateTransactionStatus set parameters ok.");
}


sub test_CaptureTransactionsUnsuccessfulWrongStatus {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedTransaction = $self->_createLineItemPaymentSaferpay2('AUTHORIZED', '123');
    my $MockedShop = $self->_createShop($MockedTransaction);
    my $MockedForm = $self->_createForm();
    my $MockedServlet = $self->_createServlet($MockedShop, $MockedForm);
    my $MockedSaferpayService = $self->_createSaferpayService();
    my $SUT = $self->_createSUT($MockedSaferpayService, {});
    $SUT->CaptureTransactions($MockedServlet);
    my $aUpdateTransactionStatusParams = GetParametersFromMockifyCall($MockedTransaction, 'updateTransactionStatus');
    my $aExpectedParameters = [$MockedTransaction->get('TransID'), 'CAPTURED', 'CAPTURED'];
    is_deeply($aUpdateTransactionStatusParams, $aExpectedParameters, "$SubTestName - updateTransactionStatus set parameters ok.");
}


sub test_CaptureTransactionsUnsuccessfulAlreadyCaptured {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedTransaction = $self->_createLineItemPaymentSaferpay2('CAPTURED', '123');
    my $MockedShop = $self->_createShop($MockedTransaction);
    my $MockedForm = $self->_createForm();
    my $MockedServlet = $self->_createServlet($MockedShop, $MockedForm);
    my $MockedSaferpayService = $self->_createSaferpayService();
    my $SUT = $self->_createSUT($MockedSaferpayService, {});
    $SUT->CaptureTransactions($MockedServlet);
    my $aUpdateTransactionStatusParams = GetParametersFromMockifyCall($MockedForm, 'addFormError');
    my $hExpectedParameters = [{
        Reason => 'ALREADY_CAPTURED',
        TransactionId => $MockedTransaction->get('TransID'),
        LoopName => 'ListObjectIDs',
        Form => 'Transactions',
        Name => 'ListObjectID'
    }];
    is_deeply($aUpdateTransactionStatusParams, $hExpectedParameters, "$SubTestName - updateTransactionStatus set parameters ok.");
}

sub test_CaptureTransactionsUnsuccessfulIdNone {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedTransaction = $self->_createLineItemPaymentSaferpay2('FAILED', 'NONE');
    my $MockedShop = $self->_createShop($MockedTransaction);
    my $MockedForm = $self->_createForm();
    my $MockedServlet = $self->_createServlet($MockedShop, $MockedForm);
    my $MockedSaferpayService = $self->_createSaferpayService();
    my $SUT = $self->_createSUT($MockedSaferpayService, {});
    $SUT->CaptureTransactions($MockedServlet);
    my $aUpdateTransactionStatusParams = GetParametersFromMockifyCall($MockedForm, 'addFormError');
    my $hExpectedParameters = [{
        Reason => 'INVALID_TRANSACTION_ID',
        TransactionId => $MockedTransaction->get('TransID'),
        LoopName => 'ListObjectIDs',
        Form => 'Transactions',
        Name => 'ListObjectID'
    }];
    is_deeply($aUpdateTransactionStatusParams, $hExpectedParameters, "$SubTestName - updateTransactionStatus set parameters ok.");
}



sub _createForm {
    my $self = shift;

    my $Mockify = Test::Mockify->new( 'DE_EPAGES::Presentation::API::Form');
    $Mockify->mock('executeFormError')->when()->thenReturnUndef();
    $Mockify->mock('addFormError')->when(HashRef())->thenReturnUndef();
    $Mockify->mock('value')->when(String('ListObjectID'))->thenReturnArray([1]);
    return $Mockify->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ( $Shop, $Form ) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    my $hServletParams = {
        'OrderId'      => '123',
        'LineItemPaymentGUID'    => '123456'
    };
    $MockifyServlet->addParams($hServletParams);
    $MockifyServlet->mock('writeContentOutput', sub{
        shift;
        $MockifyServlet->{'writeContentOutputParams_MockVar'} = \@_;
    });
    $MockifyServlet->addMockWithReturnValue( 'form', $Form );
    return $MockifyServlet->getMockObject();
}


sub _createOrder {
    my $self = shift;

    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer(),
        'Alias'             => '42',
        'ID'                => '123'
    };

    my $MockifyCustomerOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes
    );

    return $MockifyCustomerOrder->getMockObject();
}


sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'Payment'    => $self->_createLineItemPaymentSaferpay2(),
        'CurrencyID' => 'EUR',
        'GrandTotal' => '10'
    };
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes
    );

    return $MockifyContainer->getMockObject();
}


sub _createLineItemPaymentSaferpay2 {
    my $self = shift;
    my ($TransStatus, $TransID) = @_;

    my $hAttributes = {
        'PaymentMethod' => $self->_createPaymentMethod(),
        'GUID'          => '123456',
        'TransStatus'   => $TransStatus,
        'TransID'       => $TransID,
        'TransID2'      => SAFERPAY_INITIALIZE_TOKEN,
        'IsLiabilityShifted' => undef,
    };
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Saferpay2::API::Object::LineItemPaymentSaferpay2', $hAttributes
    );
    $MockifyContainer->addMockWithReturnValueAndParameterCheck('updateTransactionStatus', undef, ['string', 'string', 'string']);
    $MockifyContainer->addMockWithReturnValueAndParameterCheck('updateLiabilityShift', undef, ['int']);
    return $MockifyContainer->getMockObject();
}


sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        'IsDirectCapture' => 0 ###
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Saferpay2::API::Object::PaymentMethodSaferpay2', $hAttributes
    );
    return $MockifyPaymentMethod->getMockObject();
}


sub _createShop {
    my $self = shift;
    my ( $MockedObject ) = @_;

    my $Attributes = {
        'IsUnityRegistered' => 0
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $Attributes);
    my $aParams = ['string', 'string'];
    $MockifyShop->addMockWithReturnValueAndParameterCheck('loadObject', $MockedObject, $aParams);

    return $MockifyShop->getMockObject();
}

sub _createSUT {
    my $self = shift;
    my ( $SaferpayService, $hHandleTransactionResponse ) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::Saferpay2::UI::Shop');
    $MockifySut->mock('_LogPayment')->when(String(), HashRef() )->thenReturnUndef();
    $MockifySut->mockStatic('DE_EPAGES::Saferpay2::API::SaferpayService::new')->whenAny()->thenReturn($SaferpayService);
    $MockifySut->mockImported(
        'DE_EPAGES::Saferpay2::API::Tools', 'HandleTransactionResponse'
    )->whenAny()->thenReturn($hHandleTransactionResponse);

    return $MockifySut->getMockObject();
}

sub _createSaferpayService {
    my $self = shift;
    my ( $hCaptureResponse ) = @_;

    $hCaptureResponse //=  {Status => 'CAPTURED'};
    my $Mockify = Test::Mockify->new('DE_EPAGES::Saferpay2::API::SaferpayService');

    $Mockify->mock('CaptureTransaction')->when(Number())->thenReturn($hCaptureResponse);
    $Mockify->mock('DoAssertCall')->whenAny()->thenReturnUndef();

    return $Mockify->getMockObject();
}


#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
